@@ -69,6 +69,10 @@
 # define DISTRHO_PLUGIN_WANT_MIDI_OUTPUT 0
 #endif
 
+#ifndef DISTRHO_PLUGIN_WANT_PARAMETER_VALUE_CHANGE_REQUEST
+# define DISTRHO_PLUGIN_WANT_PARAMETER_VALUE_CHANGE_REQUEST 0
+#endif
+
 #ifndef DISTRHO_PLUGIN_WANT_PROGRAMS
 # define DISTRHO_PLUGIN_WANT_PROGRAMS 0
 #endif
@@ -77,8 +81,13 @@
 # define DISTRHO_PLUGIN_WANT_STATE 0
 #endif
 
+#ifndef DISTRHO_PLUGIN_WANT_STATEFILES
+# define DISTRHO_PLUGIN_WANT_STATEFILES 0
+#endif
+
 #ifndef DISTRHO_PLUGIN_WANT_FULL_STATE
 # define DISTRHO_PLUGIN_WANT_FULL_STATE 0
+# define DISTRHO_PLUGIN_WANT_FULL_STATE_WAS_NOT_SET
 #endif
 
 #ifndef DISTRHO_PLUGIN_WANT_TIMEPOS
@@ -108,7 +117,7 @@
 // Define DISTRHO_UI_URI if needed
 
 #ifndef DISTRHO_UI_URI
-# define DISTRHO_UI_URI DISTRHO_PLUGIN_URI "#UI"
+# define DISTRHO_UI_URI DISTRHO_PLUGIN_URI "#DPF_UI"
 #endif
 
 // -----------------------------------------------------------------------
@@ -127,11 +136,19 @@
 # error Synths need MIDI input to work!
 #endif
 
+// -----------------------------------------------------------------------
+// Enable state if plugin wants state files
+
+#if DISTRHO_PLUGIN_WANT_STATEFILES && ! DISTRHO_PLUGIN_WANT_STATE
+# undef DISTRHO_PLUGIN_WANT_STATE
+# define DISTRHO_PLUGIN_WANT_STATE 1
+#endif
+
 // -----------------------------------------------------------------------
 // Enable full state if plugin exports presets
 
-#if DISTRHO_PLUGIN_WANT_PROGRAMS && DISTRHO_PLUGIN_WANT_STATE && ! DISTRHO_PLUGIN_WANT_FULL_STATE
-# warning Plugins with programs and state need to implement full state API
+#if DISTRHO_PLUGIN_WANT_PROGRAMS && DISTRHO_PLUGIN_WANT_STATE && defined(DISTRHO_PLUGIN_WANT_FULL_STATE_WAS_NOT_SET)
+# warning Plugins with programs and state should implement full state API too
 # undef DISTRHO_PLUGIN_WANT_FULL_STATE
 # define DISTRHO_PLUGIN_WANT_FULL_STATE 1
 #endif
@@ -149,6 +166,13 @@
 # define DISTRHO_PLUGIN_HAS_UI 0
 #endif
 
+// -----------------------------------------------------------------------
+// Prevent users from messing about with DPF internals
+
+#ifdef DISTRHO_UI_IS_STANDALONE
+# error DISTRHO_UI_IS_STANDALONE must not be defined
+#endif
+
 // -----------------------------------------------------------------------
 
 #endif // DISTRHO_PLUGIN_CHECKS_H_INCLUDED
