@@ -13,12 +13,18 @@ rm -rf repos
 mkdir repos
 cd repos
 
-git clone --depth 1 git://github.com/DISTRHO/DPF
+git clone --depth 1 --recursive -b develop git://github.com/DISTRHO/DPF
 
 for PLUGIN in ${PLUGINS[@]}; do
   git clone --depth 1 git://github.com/DISTRHO/$PLUGIN
 done
 
+# special case for ProM
+cd ProM
+git submodule init
+git submodule update
+cd ..
+
 cd ..
 
 rm -rf dpf
