@@ -135,7 +135,6 @@ DistrhoPluginNekobi::DistrhoPluginNekobi()
     fParams.decay  = 75.0f;
     fParams.accent = 25.0f;
     fParams.volume = 75.0f;
-    fParams.bypass = false;
 
     // Internal stuff
     fSynth.waveform  = 0.0f;
@@ -164,7 +163,7 @@ void DistrhoPluginNekobi::initParameter(uint32_t index, Parameter& parameter)
     switch (index)
     {
     case paramWaveform:
-        parameter.hints      = kParameterIsAutomable|kParameterIsInteger;
+        parameter.hints      = kParameterIsAutomatable|kParameterIsInteger;
         parameter.name       = "Waveform";
         parameter.symbol     = "waveform";
         parameter.ranges.def = 0.0f;
@@ -183,7 +182,7 @@ void DistrhoPluginNekobi::initParameter(uint32_t index, Parameter& parameter)
         }
         break;
     case paramTuning:
-        parameter.hints      = kParameterIsAutomable; // was 0.5 <-> 2.0, log
+        parameter.hints      = kParameterIsAutomatable; // was 0.5 <-> 2.0, log
         parameter.name       = "Tuning";
         parameter.symbol     = "tuning";
         parameter.ranges.def = 0.0f;
@@ -192,7 +191,7 @@ void DistrhoPluginNekobi::initParameter(uint32_t index, Parameter& parameter)
         parameter.midiCC = 75;
         break;
     case paramCutoff:
-        parameter.hints      = kParameterIsAutomable; // modified x2.5
+        parameter.hints      = kParameterIsAutomatable; // modified x2.5
         parameter.name       = "Cutoff";
         parameter.symbol     = "cutoff";
         parameter.unit       = "%";
@@ -202,7 +201,7 @@ void DistrhoPluginNekobi::initParameter(uint32_t index, Parameter& parameter)
         parameter.midiCC = 74;
         break;
     case paramResonance:
-        parameter.hints      = kParameterIsAutomable; // modified x100
+        parameter.hints      = kParameterIsAutomatable; // modified x100
         parameter.name       = "VCF Resonance";
         parameter.symbol     = "resonance";
         parameter.unit       = "%";
@@ -212,7 +211,7 @@ void DistrhoPluginNekobi::initParameter(uint32_t index, Parameter& parameter)
         parameter.midiCC = 71;
         break;
     case paramEnvMod:
-        parameter.hints      = kParameterIsAutomable; // modified x100
+        parameter.hints      = kParameterIsAutomatable; // modified x100
         parameter.name       = "Env Mod";
         parameter.symbol     = "env_mod";
         parameter.unit       = "%";
@@ -222,7 +221,7 @@ void DistrhoPluginNekobi::initParameter(uint32_t index, Parameter& parameter)
         parameter.midiCC = 1; //Mod Wheel
         break;
     case paramDecay:
-        parameter.hints      = kParameterIsAutomable; // was 0.000009 <-> 0.0005, log
+        parameter.hints      = kParameterIsAutomatable; // was 0.000009 <-> 0.0005, log
         parameter.name       = "Decay";
         parameter.symbol     = "decay";
         parameter.unit       = "%";
@@ -232,7 +231,7 @@ void DistrhoPluginNekobi::initParameter(uint32_t index, Parameter& parameter)
         parameter.midiCC = 72;
         break;
     case paramAccent:
-        parameter.hints      = kParameterIsAutomable; // modified x100
+        parameter.hints      = kParameterIsAutomatable; // modified x100
         parameter.name       = "Accent";
         parameter.symbol     = "accent";
         parameter.unit       = "%";
@@ -242,7 +241,7 @@ void DistrhoPluginNekobi::initParameter(uint32_t index, Parameter& parameter)
         parameter.midiCC = 76;
         break;
     case paramVolume:
-        parameter.hints      = kParameterIsAutomable; // modified x100
+        parameter.hints      = kParameterIsAutomatable; // modified x100
         parameter.name       = "Volume";
         parameter.symbol     = "volume";
         parameter.unit       = "%";
@@ -251,9 +250,6 @@ void DistrhoPluginNekobi::initParameter(uint32_t index, Parameter& parameter)
         parameter.ranges.max = 100.0f;
         parameter.midiCC = 7; //Volume
         break;
-    case paramBypass:
-        parameter.initDesignation(kParameterDesignationBypass);
-        break;
     }
 }
 
@@ -280,8 +276,6 @@ float DistrhoPluginNekobi::getParameterValue(uint32_t index) const
         return fParams.accent;
     case paramVolume:
         return fParams.volume;
-    case paramBypass:
-        return fParams.bypass ? 1.0f : 0.0f;
     }
 
     return 0.0f;
@@ -331,14 +325,6 @@ void DistrhoPluginNekobi::setParameterValue(uint32_t index, float value)
         fSynth.volume = value/100.0f;
         DISTRHO_SAFE_ASSERT(fSynth.volume >= 0.0f && fSynth.volume <= 1.0f);
         break;
-    case paramBypass: {
-        const bool bypass = (value > 0.5f);
-        if (fParams.bypass != bypass)
-        {
-            fParams.bypass = bypass;
-            nekobee_synth_all_voices_off(&fSynth);
-        }
-    }   break;
     }
 }
 
@@ -374,10 +360,6 @@ void DistrhoPluginNekobi::run(const float**, float** outputs, uint32_t frames, c
         return;
     }
 
-    // ignore midi input if bypassed
-    if (fParams.bypass)
-        midiEventCount = 0;
-
     while (framesDone < frames)
     {
         if (fSynth.nugget_remains == 0)
